@@ -11,19 +11,22 @@ import sys
 import types
 import warnings
 
-from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning)
+from ruamel.yaml.error import (MarkedYAMLError, MarkedYAMLFutureWarning,
+                               MantissaNoDotYAML1_1Warning)
 from ruamel.yaml.nodes import *                               # NOQA
 from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
 from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
                                 ordereddict, text_type, nprint, version_tnf)
 from ruamel.yaml.comments import *                               # NOQA
 from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
-                                  CommentedKeySeq, CommentedSeq)
+                                  CommentedKeySeq, CommentedSeq, TaggedScalar)
 from ruamel.yaml.scalarstring import *                           # NOQA
 from ruamel.yaml.scalarstring import (PreservedScalarString, SingleQuotedScalarString,
                                       DoubleQuotedScalarString, ScalarString)
 from ruamel.yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
+from ruamel.yaml.scalarfloat import ScalarFloat
 from ruamel.yaml.timestamp import TimeStamp
+from ruamel.yaml.util import RegExp
 
 if False:  # MYPY
     from typing import Any, Dict, List, Set, Generator  # NOQA
@@ -411,8 +414,8 @@ class SafeConstructor(BaseConstructor):
 
     def construct_yaml_float(self, node):
         # type: (Any) -> float
-        value_s = to_str(self.construct_scalar(node))
-        value_s = value_s.replace('_', '').lower()
+        value_so = to_str(self.construct_scalar(node))
+        value_s = value_so.replace('_', '').lower()
         sign = +1
         if value_s[0] == '-':
             sign = -1
@@ -422,7 +425,7 @@ class SafeConstructor(BaseConstructor):
             return sign * self.inf_value
         elif value_s == '.nan':
             return self.nan_value
-        elif ':' in value_s:
+        elif self.resolver.processing_version != (1, 2) and ':' in value_s:
             digits = [float(part) for part in value_s.split(':')]
             digits.reverse()
             base = 1
@@ -432,6 +435,11 @@ class SafeConstructor(BaseConstructor):
                 base *= 60
             return sign * value
         else:
+            if self.resolver.processing_version != (1, 2) and 'e' in value_s:
+                # value_s is lower case independent of input
+                mantissa, exponent = value_s.split('e')
+                if '.' not in mantissa:
+                    warnings.warn(MantissaNoDotYAML1_1Warning(node, value_so))
             return sign * float(value_s)
 
     if PY3:
@@ -464,7 +472,7 @@ class SafeConstructor(BaseConstructor):
                     None, None,
                     "failed to decode base64 data: %s" % exc, node.start_mark)
 
-    timestamp_regexp = re.compile(
+    timestamp_regexp = RegExp(
         u'''^(?P<year>[0-9][0-9][0-9][0-9])
           -(?P<month>[0-9][0-9]?)
           -(?P<day>[0-9][0-9]?)
@@ -479,7 +487,15 @@ class SafeConstructor(BaseConstructor):
     def construct_yaml_timestamp(self, node, values=None):
         # type: (Any, Any) -> Any
         if values is None:
-            match = self.timestamp_regexp.match(node.value)
+            try:
+                match = self.timestamp_regexp.match(node.value)
+            except TypeError:
+                match = None
+            if match is None:
+                raise ConstructorError(
+                    None, None,
+                    'failed to construct timestamp from "{}"'.format(node.value),
+                    node.start_mark)
             values = match.groupdict()
         year = int(values['year'])
         month = int(values['month'])
@@ -1034,6 +1050,68 @@ class RoundTripConstructor(SafeConstructor):
         else:
             return sign * int(value_s)
 
+    def construct_yaml_float(self, node):
+        # type: (Any) -> Any
+        def leading_zeros(v):
+            # type: (Any) -> int
+            lead0 = 0
+            idx = 0
+            while idx < len(v) and v[idx] in '0.':
+                if v[idx] == '0':
+                    lead0 += 1
+                idx += 1
+            return lead0
+        # underscore = None
+        m_sign = False  # type: Any
+        value_so = to_str(self.construct_scalar(node))
+        value_s = value_so.replace('_', '').lower()
+        sign = +1
+        if value_s[0] == '-':
+            sign = -1
+        if value_s[0] in '+-':
+            m_sign = value_s[0]
+            value_s = value_s[1:]
+        if value_s == '.inf':
+            return sign * self.inf_value
+        if value_s == '.nan':
+            return self.nan_value
+        if self.resolver.processing_version != (1, 2) and ':' in value_s:
+            digits = [float(part) for part in value_s.split(':')]
+            digits.reverse()
+            base = 1
+            value = 0.0
+            for digit in digits:
+                value += digit * base
+                base *= 60
+            return sign * value
+        if 'e' in value_s:
+            try:
+                mantissa, exponent = value_so.split('e')
+                exp = 'e'
+            except ValueError:
+                mantissa, exponent = value_so.split('E')
+                exp = 'E'
+            if self.resolver.processing_version != (1, 2):
+                # value_s is lower case independent of input
+                if '.' not in mantissa:
+                    warnings.warn(MantissaNoDotYAML1_1Warning(node, value_so))
+            lead0 = leading_zeros(mantissa)
+            width = len(mantissa)
+            prec = mantissa.find('.')
+            if m_sign:
+                width -= 1
+            e_width = len(exponent)
+            e_sign = exponent[0] in '+-'
+            # print('sf', width, prec, m_sign, exp, e_width, e_sign)
+            return ScalarFloat(sign * float(value_s),  # type: ignore
+                               width=width, prec=prec, m_sign=m_sign,
+                               m_lead0=lead0, exp=exp, e_width=e_width, e_sign=e_sign)
+        width = len(value_so)
+        prec = value_so.index('.')  # you can use index, this would not be float without dot
+        lead0 = leading_zeros(value_so)
+        return ScalarFloat(sign * float(value_s),  # type: ignore
+                           width=width, prec=prec, m_sign=m_sign, m_lead0=lead0)
+
     def construct_yaml_str(self, node):
         # type: (Any) -> Any
         value = self.construct_scalar(node)
@@ -1342,25 +1420,43 @@ class RoundTripConstructor(SafeConstructor):
     def construct_undefined(self, node):
         # type: (Any) -> Any
         try:
-            data = CommentedMap()
-            data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
-            if node.flow_style is True:
-                data.fa.set_flow_style()
-            elif node.flow_style is False:
-                data.fa.set_block_style()
-            data.yaml_set_tag(node.tag)
-            yield data
-            self.construct_mapping(node, data)
-        except:
-            raise ConstructorError(
-                None, None,
-                "could not determine a constructor for the tag %r" %
-                utf8(node.tag),
-                node.start_mark)
+            if isinstance(node, MappingNode):
+                data = CommentedMap()
+                data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
+                if node.flow_style is True:
+                    data.fa.set_flow_style()
+                elif node.flow_style is False:
+                    data.fa.set_block_style()
+                data.yaml_set_tag(node.tag)
+                yield data
+                self.construct_mapping(node, data)
+                return
+            elif isinstance(node, ScalarNode):
+                data = TaggedScalar()
+                data.value = self.construct_scalar(node)
+                data.style = node.style
+                data.yaml_set_tag(node.tag)
+                yield data
+                return
+        except:  # NOQA
+            pass
+        raise ConstructorError(
+            None, None,
+            "could not determine a constructor for the tag %r" %
+            utf8(node.tag),
+            node.start_mark)
 
     def construct_yaml_timestamp(self, node, values=None):
         # type: (Any, Any) -> Any
-        match = self.timestamp_regexp.match(node.value)
+        try:
+            match = self.timestamp_regexp.match(node.value)
+        except TypeError:
+            match = None
+        if match is None:
+            raise ConstructorError(
+                None, None,
+                'failed to construct timestamp from "{}"'.format(node.value),
+                node.start_mark)
         values = match.groupdict()
         if not values['hour']:
             return SafeConstructor.construct_yaml_timestamp(self, node, values)
